const flatten = require('flatten') // Flatten Objects

function has (object, key) {
  /*
   * Check if object has a key
   */
  return object ? hasOwnProperty.call(object, key) : false
}

function findKeyRecursive (object, keytofind, ret = []) {
  /*
   * Finds all keys in a nested object and returns them as an array
   */

  let value

  Object.keys(object).forEach(function (key) {
    if (key === keytofind) {
      ret.push(object[key])
      return true
    }
    if (object[key] && typeof object[key] === 'object') {
      value = findKeyRecursive(object[key], keytofind, ret)
      return value !== undefined
    } else {
      return true
    }
  })
  return flatten(ret)
}

function countReferences (features) {
  /*
   * Count the references in a feature
   */

  let count = 0

  for (let idx = 0; idx < features.length; idx++) {
    try {
      count = count + features[idx]._attributes.ref.trim().split(' ').length
    } catch (err) {
      count = count + 1
    }
  }

  return count
}

function searchInElements (elements, query) {
  /*
   * Searches for a query in an array of elemts
   */

  let results = []

  try {
    var reContent = new RegExp(query.content)
  } catch (err) {
    console.log(arguments.callee) // eslint-disable-line no-caller
    console.log('>> Error while searching elements')
    return results
  }

  for (let idx = 0; idx < elements.length; idx++) {
    var matchContent = reContent.test(elements[idx]._text)

    if (matchContent) {
      results.push(elements[idx])
    }
  }

  return results
}

function searchInDocument (document, query) {
  /*
   * Search for text in (js) document elements
   */

  let elems = findKeyRecursive(document, query.element)
  let results = searchInElements(elems, query)

  if (results.length > 0) {
    document['results'] = results
    document['hits'] = undefined
  } else {
    return null
  }

  return document

}

function searchInFeatures (features, query) {
  /*
   * Searches for a query in an array of features
   * Attributes must be in _attributes
   */

  let results = []

  try {
    var reCat = new RegExp(query.category)
    var reType = new RegExp(query.type)
    var reSub = new RegExp(query.subtype)
  } catch (err) {
    console.log(arguments.callee) // eslint-disable-line no-caller
    console.log('>> Error while searching features')
    return results
  }

  for (let idx = 0; idx < features.length; idx++) {
    var matchCat = reCat.test(features[idx]._attributes.category)
    var matchType = reType.test(features[idx]._attributes.type)
    var matchSub = reSub.test(features[idx]._attributes.subtype)

    if (matchCat && matchType && matchSub) {
      results.push(features[idx])
    }
  }
  return results
}

function searchFeaturesInDocument (document, query) {
  /*
   * Search for features in (js) document
   */

  if (typeof query.category === 'undefined') { query.category = '.*' }
  if (typeof query.type === 'undefined') { query.type = '.*' }
  if (typeof query.subtype === 'undefined') { query.subtype = '.*' }

  let features = findKeyRecursive(document, 'feature')
  let results = searchInFeatures(features, query)

  if (results.length > 0) {
    document['hits'] = countReferences(results)
    document['results'] = results
  } else {
    return null
  }

  return document
}

module.exports = {
  features: function (obj, query) {
    return searchFeaturesInDocument(obj, query)
  },
  search: function (obj, query) {
    return searchInDocument(obj, query)
  }
}
